/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/motioncomp/motioncomp.hh"

#include "libvideocoding/motioncomp/motioncomp_scalar.hh"
#include "config.h"

#if ENABLE_MMX
#include "libvideocoding/motioncomp/motioncomp_mmx2.hh"
#endif

#include <libvideogfx/arch/cpu.hh>

namespace videocoding {
  using namespace videogfx;

  MotionCompensation* GetMotionCompensation(int blkw,              // block size
					    bool avg,              // do averaging with destination
					    fixed16_16 resolution, // supported MV accuracy
					    bool accurate)         // accurate calculation vs. fast calculation
  {
    CPU_Capabilities* cpu = CPU_Capabilities::AskCapabilities();

#if ENABLE_MMX
    if (cpu->AskArchitecture()==CPUArch_X86 && cpu->HasMMX2())
      {
	if ((blkw==8 || blkw == 16) && resolution >= HalfPelDist && accurate==false)
	  {
	    MotionCompensation_MMX2* mc=NULL;
	    if (blkw== 8 && !avg) mc = new MotionCompensation_MMX2_8;
	    if (blkw== 8 &&  avg) mc = new MotionCompensation_MMX2_8avg;
	    if (blkw==16 && !avg) mc = new MotionCompensation_MMX2_16;
	    if (blkw==16 &&  avg) mc = new MotionCompensation_MMX2_16avg;

	    return mc;
	  }
      }
#endif

    if (resolution >= HalfPelDist)
      {
	MotionCompensation_Scalar* mc = new MotionCompensation_Scalar(blkw,avg);
	return mc;
      }

    throw Excpt_Text(ErrSev_Error,"no appropriate motion compensation code found");
    return NULL;
  }

}
