/********************************************************************************
  mpeg2/syntax/demux.hh
    PES stream demultiplexer.

  purpose:

  notes:

  to do:

  author(s):
   - Dirk Farin, farin@ti.uni-mannheim.de

  modifications:
   03/Mar/2002 - Dirk Farin
     - new interface definition
 ********************************************************************************
    Copyright (C) 2002  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_DEMUX_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_DEMUX_HH

#include <libvideocoding/mpeg2/syntax/pessource.hh>
#include <libvideogfx/containers/queue.hh>

namespace videocoding {
  using namespace videogfx;

  class MP2_PES_Demux
  {
  public:
    MP2_PES_Demux();
    ~MP2_PES_Demux();


    /* initialization */

    void SetPESSource(MP2_PES_Source&);
    void Reset(); /* Call after you changed the current position in the input stream. */


    /* configuration (may be changed during usage) */

    // To be able to get packet of a channel, you have to enable this channel
    // before decoding.
    void EnableChannel(MP2_StartCode StreamID,bool enable=true);
    void EnableSystemHeaders(bool enable=true);


    /* usage */

    // Get next packet or return NULL if the end of the stream is reached.
    MP2_PES_Packet* AskNextPacket(MP2_StartCode StreamID);
    MP2_PES_Packet* GetNextPacket(MP2_StartCode StreamID);

    bool PacketPending(MP2_StartCode StreamID) const; /* True iff a packet is pending in the output queue.
							 A return value of "false" does NOT mean that
							 there will not be more packets of this channel
							 in the stream. */

    // The same as for PES packets; just for system packets.
    MP2_SystemPacket* AskNextSystemPacket();
    MP2_SystemPacket* GetNextSystemPacket();
    bool SystemPacketPending() const;

    bool ReadAPacket(); /* Read packet from the input. Returns "false" if there are
			   no more packets. The read packet will be stored in the
			   appropriate output queue if it is enabled. */

  private:
#define c_FirstStream 0xBC
#define c_LastStream  0xFF

    MP2_PES_Source* d_src;
    Queue<MP2_PES_Packet*> d_outputqueue[c_LastStream-c_FirstStream+1];
    Queue<MP2_SystemPacket*> d_miscqueue;

    bool d_enabled[c_LastStream-c_FirstStream+1];
    bool d_system_headers_enabled;
  };



  class MP2_PES_DemuxExtractor : public MP2_PES_Source
  {
  public:
    MP2_PES_DemuxExtractor();

    void SetSource(MP2_PES_Demux& src) { d_demux=&src; }
    void SetChannel(MP2_StartCode ch) { d_channel=ch; d_initialized=true; }

    MP2_SystemPacket* GetNextPacket();
    MP2_SystemPacket* AskNextPacket();
    bool IsEOF() const;
    bool PacketPending() const;

  private:
    MP2_PES_Demux* d_demux;
    MP2_StartCode  d_channel;
    bool d_initialized;  // d_channel contains valid information
  };

}

#endif
